@@ -43,7 +43,9 @@ class CitationGraphDataset(DGLBuiltinDataset):
     force_reload : bool
         Whether to reload the dataset. Default: False
     verbose: bool
-      Whether to print out progress information. Default: True.
+        Whether to print out progress information. Default: True.
+    reverse_edge: bool
+        Whether to add reverse edges in graph. Default: True.
     """
     _urls = {
         'cora_v2' : 'dataset/cora_v2.zip',
@@ -51,7 +53,7 @@ class CitationGraphDataset(DGLBuiltinDataset):
         'pubmed' : 'dataset/pubmed.zip',
     }
 
-    def __init__(self, name, raw_dir=None, force_reload=False, verbose=True):
+    def __init__(self, name, raw_dir=None, force_reload=False, verbose=True, reverse_edge=True):
         assert name.lower() in ['cora', 'citeseer', 'pubmed']
 
         # Previously we use the pre-processing in pygcn (https://github.com/tkipf/pygcn)
@@ -60,6 +62,8 @@ class CitationGraphDataset(DGLBuiltinDataset):
             name = 'cora_v2'
 
         url = _get_dgl_url(self._urls[name])
+        self._reverse_edge = reverse_edge
+
         super(CitationGraphDataset, self).__init__(name,
                                                    url=url,
                                                    raw_dir=raw_dir,
@@ -104,7 +108,11 @@ class CitationGraphDataset(DGLBuiltinDataset):
 
         features = sp.vstack((allx, tx)).tolil()
         features[test_idx_reorder, :] = features[test_idx_range, :]
-        graph = nx.DiGraph(nx.from_dict_of_lists(graph))
+
+        if self.reverse_edge:
+            graph = nx.DiGraph(nx.from_dict_of_lists(graph))
+        else:
+            graph = nx.Graph(nx.from_dict_of_lists(graph))
 
         onehot_labels = np.vstack((ally, ty))
         onehot_labels[test_idx_reorder, :] = onehot_labels[test_idx_range, :]
@@ -114,16 +122,16 @@ class CitationGraphDataset(DGLBuiltinDataset):
         idx_train = range(len(y))
         idx_val = range(len(y), len(y)+500)
 
-        train_mask = _sample_mask(idx_train, labels.shape[0])
-        val_mask = _sample_mask(idx_val, labels.shape[0])
-        test_mask = _sample_mask(idx_test, labels.shape[0])
+        train_mask = generate_mask_tensor(_sample_mask(idx_train, labels.shape[0]))
+        val_mask = generate_mask_tensor(_sample_mask(idx_val, labels.shape[0]))
+        test_mask = generate_mask_tensor(_sample_mask(idx_test, labels.shape[0]))
 
         self._graph = graph
         g = from_networkx(graph)
 
-        g.ndata['train_mask'] = generate_mask_tensor(train_mask)
-        g.ndata['val_mask'] = generate_mask_tensor(val_mask)
-        g.ndata['test_mask'] = generate_mask_tensor(test_mask)
+        g.ndata['train_mask'] = train_mask
+        g.ndata['val_mask'] = val_mask
+        g.ndata['test_mask'] = test_mask
         g.ndata['label'] = F.tensor(labels)
         g.ndata['feat'] = F.tensor(_preprocess_features(features), dtype=F.data_type_dict['float32'])
         self._num_classes = onehot_labels.shape[1]
@@ -171,20 +179,22 @@ class CitationGraphDataset(DGLBuiltinDataset):
         graphs, _ = load_graphs(str(graph_path))
 
         info = load_info(str(info_path))
-        self._g = graphs[0]
+        graph = graphs[0]
+        self._g = graph
+        # for compatability
         graph = graph.clone()
-        graph.pop('train_mask')
-        graph.pop('val_mask')
-        graph.pop('test_mask')
-        graph.pop('feat')
-        graph.pop('label')
+        graph.ndata.pop('train_mask')
+        graph.ndata.pop('val_mask')
+        graph.ndata.pop('test_mask')
+        graph.ndata.pop('feat')
+        graph.ndata.pop('label')
         graph = to_networkx(graph)
         self._graph = nx.DiGraph(graph)
 
         self._num_classes = info['num_classes']
-        self._g.ndata['train_mask'] = generate_mask_tensor(self._g.ndata['train_mask'].numpy())
-        self._g.ndata['val_mask'] = generate_mask_tensor(self._g.ndata['val_mask'].numpy())
-        self._g.ndata['test_mask'] = generate_mask_tensor(self._g.ndata['test_mask'].numpy())
+        self._g.ndata['train_mask'] = generate_mask_tensor(F.asnumpy(self._g.ndata['train_mask']))
+        self._g.ndata['val_mask'] = generate_mask_tensor(F.asnumpy(self._g.ndata['val_mask']))
+        self._g.ndata['test_mask'] = generate_mask_tensor(F.asnumpy(self._g.ndata['test_mask']))
         # hack for mxnet compatability
 
         if self.verbose:
@@ -252,6 +262,11 @@ class CitationGraphDataset(DGLBuiltinDataset):
         deprecate_property('dataset.feat', 'g.ndata[\'feat\']')
         return self._g.ndata['feat']
 
+    @property
+    def reverse_edge(self):
+        return self._reverse_edge
+    
+
 def _preprocess_features(features):
     """Row-normalize feature matrix and convert to tuple representation"""
     rowsum = np.asarray(features.sum(1))
@@ -328,7 +343,7 @@ class CoraGraphDataset(CitationGraphDataset):
     - Number of Classes: 7
     - Label split:
 
-        - Train: 140 
+        - Train: 140
         - Valid: 500
         - Test: 1000
 
@@ -341,6 +356,8 @@ class CoraGraphDataset(CitationGraphDataset):
         Whether to reload the dataset. Default: False
     verbose: bool
         Whether to print out progress information. Default: True.
+    reverse_edge: bool
+        Whether to add reverse edges in graph. Default: True.
 
     Attributes
     ----------
@@ -383,10 +400,10 @@ class CoraGraphDataset(CitationGraphDataset):
     >>> # Train, Validation and Test
 
     """
-    def __init__(self, raw_dir=None, force_reload=False, verbose=True):
+    def __init__(self, raw_dir=None, force_reload=False, verbose=True, reverse_edge=True):
         name = 'cora'
 
-        super(CoraGraphDataset, self).__init__(name, raw_dir, force_reload, verbose)
+        super(CoraGraphDataset, self).__init__(name, raw_dir, force_reload, verbose, reverse_edge)
 
     def __getitem__(self, idx):
         r"""Gets the graph object
@@ -481,6 +498,8 @@ class CiteseerGraphDataset(CitationGraphDataset):
         Whether to reload the dataset. Default: False
     verbose: bool
         Whether to print out progress information. Default: True.
+    reverse_edge: bool
+        Whether to add reverse edges in graph. Default: True.
 
     Attributes
     ----------
@@ -526,10 +545,10 @@ class CiteseerGraphDataset(CitationGraphDataset):
     >>> # Train, Validation and Test
 
     """
-    def __init__(self, raw_dir=None, force_reload=False, verbose=True):
+    def __init__(self, raw_dir=None, force_reload=False, verbose=True, reverse_edge=True):
         name = 'citeseer'
 
-        super(CiteseerGraphDataset, self).__init__(name, raw_dir, force_reload, verbose)
+        super(CiteseerGraphDataset, self).__init__(name, raw_dir, force_reload, verbose, reverse_edge)
 
     def __getitem__(self, idx):
         r"""Gets the graph object
@@ -624,6 +643,8 @@ class PubmedGraphDataset(CitationGraphDataset):
         Whether to reload the dataset. Default: False
     verbose: bool
         Whether to print out progress information. Default: True.
+    reverse_edge: bool
+        Whether to add reverse edges in graph. Default: True.
 
     Attributes
     ----------
@@ -666,10 +687,10 @@ class PubmedGraphDataset(CitationGraphDataset):
     >>> # Train, Validation and Test
 
     """
-    def __init__(self, raw_dir=None, force_reload=False, verbose=True):
+    def __init__(self, raw_dir=None, force_reload=False, verbose=True, reverse_edge=True):
         name = 'pubmed'
 
-        super(PubmedGraphDataset, self).__init__(name, raw_dir, force_reload, verbose)
+        super(PubmedGraphDataset, self).__init__(name, raw_dir, force_reload, verbose, reverse_edge)
 
     def __getitem__(self, idx):
         r"""Gets the graph object
@@ -697,7 +718,7 @@ class PubmedGraphDataset(CitationGraphDataset):
         r"""The number of graphs in the dataset."""
         return super(PubmedGraphDataset, self).__len__()
 
-def load_cora(raw_dir=None, force_reload=False, verbose=True):
+def load_cora(raw_dir=None, force_reload=False, verbose=True, reverse_edge=True):
     """Get CoraGraphDataset
 
     Parameters
@@ -709,15 +730,17 @@ def load_cora(raw_dir=None, force_reload=False, verbose=True):
         Whether to reload the dataset. Default: False
     verbose: bool
     Whether to print out progress information. Default: True.
+    reverse_edge: bool
+        Whether to add reverse edges in graph. Default: True.
 
     Return
     -------
     CoraGraphDataset
     """
-    data = CoraGraphDataset(raw_dir, force_reload, verbose)
+    data = CoraGraphDataset(raw_dir, force_reload, verbose, reverse_edge)
     return data
 
-def load_citeseer(raw_dir=None, force_reload=False, verbose=True):
+def load_citeseer(raw_dir=None, force_reload=False, verbose=True, reverse_edge=True):
     """Get CiteseerGraphDataset
 
     Parameters
@@ -729,15 +752,17 @@ def load_citeseer(raw_dir=None, force_reload=False, verbose=True):
         Whether to reload the dataset. Default: False
     verbose: bool
     Whether to print out progress information. Default: True.
+    reverse_edge: bool
+        Whether to add reverse edges in graph. Default: True.
 
     Return
     -------
     CiteseerGraphDataset
     """
-    data = CiteseerGraphDataset(raw_dir, force_reload, verbose)
+    data = CiteseerGraphDataset(raw_dir, force_reload, verbose, reverse_edge)
     return data
 
-def load_pubmed(raw_dir=None, force_reload=False, verbose=True):
+def load_pubmed(raw_dir=None, force_reload=False, verbose=True, reverse_edge=True):
     """Get PubmedGraphDataset
 
     Parameters
@@ -749,12 +774,14 @@ def load_pubmed(raw_dir=None, force_reload=False, verbose=True):
             Whether to reload the dataset. Default: False
         verbose: bool
         Whether to print out progress information. Default: True.
+    reverse_edge: bool
+        Whether to add reverse edges in graph. Default: True.
 
     Return
     -------
     PubmedGraphDataset
     """
-    data = PubmedGraphDataset(raw_dir, force_reload, verbose)
+    data = PubmedGraphDataset(raw_dir, force_reload, verbose, reverse_edge)
     return data
 
 class CoraBinary(DGLBuiltinDataset):
